@@ -119,12 +119,12 @@ class ClientSession(object):
         self._catalog = catalog
         self._schema = schema
         self._source = source
-        self._properties = properties or {}
-        self._headers = headers or {}
+        self._properties = properties.copy() if properties is not None else {}
+        self._headers = headers.copy() if headers is not None else {}
         self._transaction_id = transaction_id
         self._extra_credential = extra_credential
-        self._client_tags = client_tags
-        self._roles = roles or {}
+        self._client_tags = client_tags.copy() if client_tags is not None else list()
+        self._roles = roles.copy() if roles is not None else {}
         self._prepared_statements: Dict[str, str] = {}
         self._object_lock = threading.Lock()
 
@@ -225,7 +225,7 @@ def get_session_property_values(headers, header):
     kvs = get_header_values(headers, header)
     return [
         (k.strip(), urllib.parse.unquote_plus(v.strip()))
-        for k, v in (kv.split("=", 1) for kv in kvs)
+        for k, v in (kv.split("=", 1) for kv in kvs if kv)
     ]
 
 
@@ -233,7 +233,7 @@ def get_prepared_statement_values(headers, header):
     kvs = get_header_values(headers, header)
     return [
         (k.strip(), urllib.parse.unquote_plus(v.strip()))
-        for k, v in (kv.split("=", 1) for kv in kvs)
+        for k, v in (kv.split("=", 1) for kv in kvs if kv)
     ]
 
 
@@ -241,7 +241,7 @@ def get_roles_values(headers, header):
     kvs = get_header_values(headers, header)
     return [
         (k.strip(), urllib.parse.unquote_plus(v.strip()))
-        for k, v in (kv.split("=", 1) for kv in kvs)
+        for k, v in (kv.split("=", 1) for kv in kvs if kv)
     ]
 
 
@@ -607,7 +607,7 @@ class TrinoRequest(object):
             for name in get_header_values(
                 http_response.headers, constants.HEADER_DEALLOCATED_PREPARE
             ):
-                self._client_session.prepared_statements.pop(name)
+                self._client_session.prepared_statements.pop(name, None)
 
         self._next_uri = response.get("nextUri")
 
